<?php
ignore_user_abort(1);
ini_set('max_execution_time', 6000);
set_time_limit(6000);
ini_set('memory_limit', '128M');
include_once 'a.charset.php';
/**
 * MultiXml class library is a PHP solution for work with XML structures via SAX
 * parser. It can be used for any XML data including YML (Yandex Market Language).
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * @author    Vadym Timofeyev <tvad@mail333.com> http://weblancer.net/users/tvv/
 * @copyright 2007 Vadym Timofeyev
 * @license   http://www.gnu.org/licenses/lgpl-3.0.txt
 * @version   1.00
 * @since     PHP 5.0
 */
abstract class MultiXml {
    /**
     * Internal stack for tags hierarchy
     * @var array
     */
    private $stack;

    /**
     * Internal stack for prepared tags hierarchy
     * @var array
     */
    private $stack_tag;

    /**
     * Internal cache
     * @var array
     */
    private $cache;

    /**
     * MultiXml config
     * @var array
     */
    protected $config;

    /**
     * Output charset
     * @var string
     */
    protected $charset;

    /**
     * Class constructor. Setup primary parameters.
     * @param array $config MultiXml config
     */
    public function __construct($config) {
        if (!($this->config = $config)) {
            throw new Exception('Required configuration structure is empty!');
        }
    }

    /**
     * Parse XML file.
     * @param string $fname Input XML file name
     * @param string $charset Output charset
     */
    public function parse($fname, $charset = 'UTF-8') {
		/*
		$gz = gzopen($fname, "r") or die("Failed to decomress $fname<br>\n");
		$fp = fopen("YML.xml", "w");
		while (!gzeof($gz)) fwrite($fp, gzread($gz, 16384));
		gzclose($gz);
		fclose($fp);
		$fname="YML.xml";
		*/
		
        if (!$fname) {
            throw new Exception('Required XML file name is empty!');
        }
        $this->charset   = $charset;
        $this->stack     = array();
        $this->stack_tag = array();
        $this->cache     = array();

        // Open XML file
        if (!($f = fopen($fname, 'rb'))) {
            throw new Exception(sprintf('Could not open XML input: %s!', $fname));
        }

        // Initialize XML parser
        $x = xml_parser_create("UTF-8");
        xml_parser_set_option($x, XML_OPTION_CASE_FOLDING, 0);
        xml_parser_set_option($x, XML_OPTION_SKIP_WHITE, 1);
        xml_set_element_handler($x, array($this, 'startElement'), array($this, 'endElement'));
        xml_set_character_data_handler($x, array($this, 'characterData'));

        // Parse XML
        $err = null;
        while ($data = fread($f, 4096)) {
		
            if (!xml_parse($x, $data, feof($f))) {
                $err = sprintf('XML error: %s at line %d', xml_error_string(xml_get_error_code($x)), xml_get_current_line_number($x));
                break;
            }
        }

        // Close opened descriptors and unset properties
        xml_parser_free($x);
        fclose($f);
        unset($this->cache);
        unset($this->stack_tag);
        unset($this->stack);

        // Check error
        if ($err) {
            throw new Exception($err);
        }
    }

    /**
     * Open Tag callback event
     * @param handle $parser Current XML Parser
     * @param string $name XML tag name
     * @param array $name XML tag attributes
     */
    protected function startElement($parser, $name, $attrs) {
        array_push($this->stack, strtolower($name));
        $tag = "/" . join("/", $this->stack);
        array_push($this->stack_tag, $tag);

        foreach ($attrs as $key => $value) {
		
            $this->cache[$tag . "/@" . strtolower($key)] = $value;
        }
    }

    /**
     * Close Tag callback event
     * @param handle $parser Current XML Parser
     * @param string $name XML tag name
     */
    protected function endElement($parser, $name) {
        array_pop($this->stack);
        $tag = array_pop($this->stack_tag);

        // Check structure
        if (isset($this->config[$tag])) {
            // Prepare data
            $arr = array();
            $iconvflag = $this->charset && $this->charset != 'UTF-8';
            foreach ($this->config[$tag] as $cols) {
			
                $s = trim(@$this->cache[$tag . ($cols[0] ? '/' . $cols[0] : '')]);
                if ($iconvflag) {
                    $s = iconv('UTF-8', $this->charset, $s);
					
                }
                $arr[] = $s;
            }
			//var_dump($arr);
            // Run callback onTagRead event
            $this->onTagRead($tag, $arr);

            // Delete unused cached data
            $arr = $this->cache;
            foreach ($arr as $key => $value) {
                if (strpos($key, $tag) === 0) {
                    unset($this->cache[$key]);
                }
            }
        }
    }

    /**
     * Character data callback event
     * @param handle $parser Current XML Parser
     * @param string $data Character data
     */
    protected function characterData($parser, $data) {
        $tag = $this->stack_tag[count($this->stack_tag) - 1];
		$data=charset_x_win($data);
        if (isset($this->cache[$tag])) {
			if ($tag=="/yml_catalog/shop/offers/offer/categoryid")
			{$this->cache[$tag] .= ",".$data;}
			else
			{$this->cache[$tag] .= $data;}
            
        } else {
            $this->cache[$tag] = $data;
        }
		//var_dump($tag);
    }

    /**
     * OnTagRead callback event
     * @param string $tag Prepared key tag
     * @param array $data Parsed data array
     */
    protected abstract function onTagRead($tag, $data);
}
?>